package com.hyc.config;

import com.alibaba.fastjson.JSONObject;
import com.hyc.domain.NodeName;
import com.hyc.domain.req.CarEventUpdate;
import com.hyc.simulate.constant.ExchangeConstant;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * rabbit更新缓存
 *
 * @author YouChe·He
 * @ClassName: RabbitConfig
 * @Description: rabbit配置
 * @CreateTime: 2024/6/16 10:32
 */
@Configuration
@Slf4j
public class MsgComment {

    @Autowired
    private NodeName nodeName;
    @Autowired
    private CacheManager cacheManager;
    @Bean
    public FanoutExchange eventUpdateExchange() {
        return new FanoutExchange(ExchangeConstant.UPDATE_CONSTANT);
    }

    @Bean
    public Queue queue() {
        return new Queue(nodeName.getNodeName());
    }

    @Bean
    public Binding binding() {
        return new Binding(nodeName.getNodeName(),
                Binding.DestinationType.QUEUE, ExchangeConstant.UPDATE_CONSTANT,
                "", null);
    }

    @Bean
    public ConnectionFactory connectionFactory() {
        CachingConnectionFactory connectionFactory = new CachingConnectionFactory();
        connectionFactory.setHost("47.103.75.98");
        connectionFactory.setPort(5672);
        connectionFactory.setPassword("guest");
        connectionFactory.setUsername("guest");
        //开启发送端确认
        connectionFactory.setPublisherConfirmType(CachingConnectionFactory.ConfirmType.CORRELATED);//SIMPLE=简单的;CORRELATED=相关的;NONE=关闭b;
        //开启消息返回
        connectionFactory.setPublisherReturns(true);
        connectionFactory.createConnection();
        return connectionFactory;
    }

    @Bean
    public RabbitAdmin rabbitAdmin(ConnectionFactory connectionFactory) {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);
        rabbitAdmin.setAutoStartup(true);
        return rabbitAdmin;
    }

    @Bean
    public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);

        rabbitTemplate.setMandatory(true);
        //消息返回
        rabbitTemplate.setReturnCallback((message, i, s, s1, s2) -> {
            log.info("消息返回实现，message：{},replyCode：{},replyText：{},exchange：{},routingKey：{}", message, i, s, s1, s2);
        });
        //发送端确认
        rabbitTemplate.setConfirmCallback((correlationData, b, s) -> {
            log.info("发送端确认，correlationData：{},ack：{},cause：{}", correlationData, b, s);

        });
        return rabbitTemplate;
    }

    @Bean
    public SimpleMessageListenerContainer messageListenerContainer(ConnectionFactory connectionFactory) {
        SimpleMessageListenerContainer simpleMessageListenerContainer = new SimpleMessageListenerContainer(connectionFactory);
        //针对哪些队列（参数为可变参数）
        simpleMessageListenerContainer.setQueueNames(nodeName.getNodeName());
        //同时有多少个消费者线程在消费这个队列，相当于线程池的线程数字。
        simpleMessageListenerContainer.setConcurrentConsumers(3);
        //最大的消费者线程数
        simpleMessageListenerContainer.setMaxConcurrentConsumers(5);
        //设置消息确认方式    NONE=不确认,MANUAL=手动确认,AUTO=自动确认;
        //自动确认
        //simpleMessageListenerContainer.setAcknowledgeMode(AcknowledgeMode.AUTO);
        //simpleMessageListenerContainer.setMessageListener(message -> log.info("springboot.rabbitmq-queue接收到的消息：[{}]", message));
        //手动确认(单条确认)
        simpleMessageListenerContainer.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        simpleMessageListenerContainer.setMessageListener((ChannelAwareMessageListener) (message, channel) -> {
            log.info("队列名:{}",nodeName.getNodeName());
            log.info("接收到的消息：[{}]",message.toString());
            String eventString = new String(message.getBody());
            CarEventUpdate carEventUpdate = JSONObject.parseObject(eventString, CarEventUpdate.class);

            Cache carEvent = cacheManager.getCache("carEvent");
            carEvent.put(carEventUpdate.getVin(),carEventUpdate.getHandleEvent());
            Cache.ValueWrapper valueWrapper = carEvent.get(carEventUpdate.getVin());
            log.info("小车{}更新后的缓存中事件是:{}",carEventUpdate.getVin(),valueWrapper.get().toString());
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        });
        //消费端限流
        simpleMessageListenerContainer.setPrefetchCount(1);
        return simpleMessageListenerContainer;
    }
}
