package com.hyc.config;

import org.springframework.amqp.core.*;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * rabbitMq配置类
 *
 * @author YouChe·He
 * @ClassName: RabbitmqConfig
 * @Description: rabbitMq配置类
 * @CreateTime: 2024/5/27 16:56
 */
@Configuration
public class RabbitmqConfig {
    public static final String FAULT_EXCHANGE = "fault_exchange";
    public static final String FAULT_MESSAGE_QUEUE = "fault_message_queue";

    public static final String FAULT_MESSAGE_ROUTINGKEY = "fault_message_router";
    public static final String INIT_CONNECT = "init_connect";
    public static final String DISCONNECT_CONNECT = "disconnect_connect";
    public static final String EXCHANGE_TOPICS_INFORM="exchange_topics_inform";

    public static final String INDEX_WARN_QUEUE = "index_warn_queue";
    /**
     * 修改指标预警配置队列
     */
    public final static String UPDATE_INDEX_WARN = "update_index_warn";


    public static final String CREATE_MQTT_CLIENT = "create_mqtt_client";

    @Bean(UPDATE_INDEX_WARN)
    public Queue UPDATE_INDEX_WARN(){
        return new Queue(UPDATE_INDEX_WARN);
    }


    @Bean(INDEX_WARN_QUEUE)
    public Queue INDEX_WARN_QUEUE(){
        return new Queue(INDEX_WARN_QUEUE);
    }

    @Bean(CREATE_MQTT_CLIENT)
    public Queue CREATE_MQTT_CLIENT(){
        return new Queue(CREATE_MQTT_CLIENT);
    }

    @Bean(EXCHANGE_TOPICS_INFORM)
    public Exchange EXCHANGE_TOPICS_INFORM(){
        //durable(true) 持久化，mq重启之后交换机还在
        return ExchangeBuilder.topicExchange(EXCHANGE_TOPICS_INFORM).durable(true).build();
    }

    //声明INIT_CONNECT队列
    @Bean(INIT_CONNECT)
    public Queue QUEUE_INFORM_EMAIL(){
        return new Queue(INIT_CONNECT);
    }
    //声明DISCONNECT_CONNECT队列
    @Bean(DISCONNECT_CONNECT)
    public Queue QUEUE_INFORM_SMS(){
        return new Queue(DISCONNECT_CONNECT);
    }



    @Bean(FAULT_EXCHANGE)
    public Exchange FAULT_EXCHANGE(){
        //durable(true) 持久化，mq重启之后交换机还在
        return ExchangeBuilder.topicExchange(FAULT_EXCHANGE).durable(true).build();
    }

    @Bean(FAULT_MESSAGE_QUEUE)
    public Queue FAULT_MESSAGE_QUEUE(){
        return new Queue(FAULT_MESSAGE_QUEUE);
    }

    @Bean
    public Binding FAULT_MESSAGE_ROUTINGKEY(@Qualifier(FAULT_MESSAGE_QUEUE) Queue queue,
                                          @Qualifier(FAULT_EXCHANGE) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(FAULT_MESSAGE_ROUTINGKEY).noargs();
    }
}
