package com.hyc.config;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hyc.domain.AnalyzeRule;
import com.hyc.domain.cache.CacheCarEvent;
import com.hyc.domain.cache.CacheIndexWarn;
import com.hyc.domain.cache.IndexWarnToRedis;
import com.hyc.kafka.demo.service.CarEventService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 项目启动同步本地缓存
 *
 * @author YouChe·He
 * @ClassName: SyncCacheRunner
 * @Description: 同步缓存Runner
 * @CreateTime: 2024/6/16 11:40
 */
@Component
public class SyncCacheRunner implements ApplicationRunner {
    @Autowired
    private CarEventService carEventService;
    @Autowired
    private CacheManager cacheManager;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public void run(ApplicationArguments args) throws Exception {

        int[] intArr = {17, 13, 11, 10, 6, 11, 6, 5, 9, 1, 2, 2, 5, 6, 5, 4, 6, 5, 8, 6, 6, 6, 2, 5, 6, 4, 4, 6, 6, 6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};

        String[] strArr = {"vin","startTime","longitude","latitude","speed","mileage","voltage","current","resistance","gear","accelerationPedal","brakePedal","fuelConsumptionRate","motorControllerTemperature","motorSpeed","motorTorque","motorTemperature","motorVoltage","motorCurrent","remainingBattery","maximumFeedbackPower","maximumDischargePower","selfCheckCounter","totalBatteryCurrent","totalBatteryVoltage","singleBatteryMaxVoltage","singleBatteryMinVoltage","singleBatteryMaxTemperature","singleBatteryMinTemperature","availableBatteryCapacity","vehicleStatus","chargingStatus","operatingStatus","socStatus","chargingEnergyStorageStatus","driveMotorStatus","positionStatus","easStatus","ptcStatus","epsStatus","absStatus","mcuStatus","heatingStatus","batteryStatus","batteryInsulationStatus","dcdcStatus","chgStatus"};
        int count = 0;
        ArrayList<AnalyzeRule> analyzeRules = new ArrayList<>();
        for (int i = 0; i < intArr.length; i++) {
            analyzeRules.add(new AnalyzeRule(count, count + intArr[i], strArr[i], "String", i > 29 ? 2 : 1));
            count = count + intArr[i];
        }


        Cache cache = cacheManager.getCache("carEvent");

        Cache analyzeRuleCache = cacheManager.getCache("analyzeRule");

        List<CacheCarEvent> cacheCarEventList = carEventService.getAllCarEvent();
        for (CacheCarEvent cacheCarEvent : cacheCarEventList) {
            redisTemplate.opsForValue().set(cacheCarEvent.getVin(), JSONObject.toJSONString(cacheCarEvent));
            cache.put(cacheCarEvent.getVin(), cacheCarEvent.getEventValue());

            redisTemplate.opsForValue().set(cacheCarEvent.getVin()+"_Analyze_Rule", JSON.toJSONString(analyzeRules));

            analyzeRuleCache.put(cacheCarEvent.getVin()+"_Analyze_Rule", JSON.toJSONString(analyzeRules));

            IndexWarnToRedis cacheIndexWarn = new IndexWarnToRedis(cacheCarEvent.getVin(), "speed,voltage", 20, 10);

            redisTemplate.opsForValue().set(cacheCarEvent.getVin()+"_Index_Warn", JSON.toJSONString(cacheIndexWarn));
            System.out.println(cacheIndexWarn.toString());



        }
        // 将数据存入 Redis 和本地缓存



    }
}
