package com.hyc.iotdbdemo.server.impl;

import com.alibaba.fastjson.JSONObject;
import com.hyc.domain.AnalyzeRule;
import com.hyc.domain.VehicleData;
import com.hyc.iotdbdemo.config.IotDBSessionConfig;
import com.hyc.iotdbdemo.model.param.IotDbParam;
import com.hyc.iotdbdemo.model.result.IotDbResult;
import com.hyc.iotdbdemo.server.IotDbServer;
import lombok.extern.log4j.Log4j2;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.iotdb.session.SessionDataSet;
import org.apache.iotdb.tsfile.read.common.Field;
import org.apache.iotdb.tsfile.read.common.RowRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.rmi.ServerException;
import java.util.*;

/**
 * description: iot服务实现类
 * date: 2022/8/15 9:43
 * author: zhouhong
 */

@Log4j2
@Service
public class IotDbServerImpl implements IotDbServer {

    @Autowired
    private CacheManager cacheManager;

    @Resource
    private IotDBSessionConfig iotDBSessionConfig;

    @Override
    public void insertData(JSONObject jsonObject) throws StatementExecutionException, ServerException, IoTDBConnectionException {

        Cache analyzeRule = cacheManager.getCache("analyzeRule");
        String vin = jsonObject.getString("vin");
        Cache.ValueWrapper valueWrapper = analyzeRule.get(vin + "_Analyze_Rule");
        String findConfigByVin = valueWrapper.get().toString();

        List<AnalyzeRule> analyzeRules = JSONObject.parseArray(findConfigByVin, AnalyzeRule.class);


        // iotDbParam: 模拟设备上报消息
        // bizkey: 业务唯一key  PK :产品唯一编码   SN:设备唯一编码
        String deviceId = "root.muyu." + jsonObject.get("vin");
        // 将设备上报的数据存入数据库(时序数据库)
        List<String> measurementsList = getInsertList(analyzeRules);
        List<String> valuesList = getInsertValueList(jsonObject,analyzeRules);
        Long startTime = jsonObject.getLong("startTime");
        iotDBSessionConfig.insertRecord(deviceId, jsonObject.getLong("startTime"), measurementsList, valuesList);
    }


    @Override
    public List<String> queryDataFromIotDb(IotDbParam vehicleData) throws Exception {
        List<String> iotDbResultList = new ArrayList<>();
        log.error("查询数据的参数是:{}", vehicleData);
        if (null != vehicleData.getVin()) {
            String sql = "select * from root.muyu." + vehicleData.getVin() + " where time >= "
                    + vehicleData.getStartTime() + " and time < " + vehicleData.getEndTime();
            SessionDataSet sessionDataSet = iotDBSessionConfig.query(sql);
            List<String> columnNames = sessionDataSet.getColumnNames();
            List<String> titleList = new ArrayList<>();
            // 排除Time字段 -- 方便后面后面拼装数据
            for (int i = 1; i < columnNames.size(); i++) {
                String[] temp = columnNames.get(i).split("\\.");
                titleList.add(temp[temp.length - 1]);
            }
            // 封装处理数据
            packagingData(vehicleData, iotDbResultList, sessionDataSet, titleList);
        } else {
            log.info("PK或者SN不能为空！！");
        }
        return iotDbResultList;
    }

    /**
     * 封装处理数据
     *
     * @param iotDbParam
     * @param iotDbResultList
     * @param sessionDataSet
     * @param titleList
     * @throws StatementExecutionException
     * @throws IoTDBConnectionException
     */
    private void packagingData(IotDbParam iotDbParam, List<String> iotDbResultList, SessionDataSet sessionDataSet, List<String> titleList)
            throws StatementExecutionException, IoTDBConnectionException {
        int fetchSize = sessionDataSet.getFetchSize();
        if (fetchSize > 0) {
            while (sessionDataSet.hasNext()) {
                VehicleData iotDbResult = new VehicleData();
                RowRecord next = sessionDataSet.next();
                List<Field> fields = next.getFields();
                long timestamp = next.getTimestamp();
//                new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(next.getTimestamp());
                iotDbResult.setStartTime(timestamp);
                Map<String, String> map = new HashMap<>();

                for (int i = 0; i < fields.size(); i++) {
                    Field field = fields.get(i);
                    // 这里的需要按照类型获取
                    map.put(titleList.get(i), field.getObjectValue(field.getDataType()).toString());
                }

                Cache analyzeRule = cacheManager.getCache("analyzeRule");
                Cache.ValueWrapper valueWrapper = analyzeRule.get(iotDbParam.getVin() + "_Analyze_Rule");
                String analyzeRuleJsonString = valueWrapper.get().toString();
                List<AnalyzeRule> analyzeRules = JSONObject.parseArray(analyzeRuleJsonString, AnalyzeRule.class);
                LinkedHashMap<String, String> stringStringLinkedHashMap = new LinkedHashMap<>();
                for (AnalyzeRule rule : analyzeRules) {

                    String s = map.get(rule.getFieldsName());
                    stringStringLinkedHashMap.put(rule.getFieldsName(),s);
                }

                iotDbResultList.add(JSONObject.toJSONString(stringStringLinkedHashMap));
            }
        }
    }

    public List<String> getInsertList(List<AnalyzeRule> analyzeRules) {


        List<String> measurementsList = new ArrayList<>();

        for (AnalyzeRule analyzeRule : analyzeRules) {
            measurementsList.add(analyzeRule.getFieldsName());
        }



//        measurementsList.add("startTime");
//        measurementsList.add("longitude");
//        measurementsList.add("latitude");
//        measurementsList.add("speed");
//        measurementsList.add("mileage");
//        measurementsList.add("voltage");
//        measurementsList.add("current");
//        measurementsList.add("resistance");
//        measurementsList.add("gear");
//        measurementsList.add("accelerationPedal");
//        measurementsList.add("brakePedal");
//        measurementsList.add("fuelConsumptionRate");
//        measurementsList.add("motorControllerTemperature");
//        measurementsList.add("motorSpeed");
//        measurementsList.add("motorTorque");
//        measurementsList.add("motorTemperature");
//        measurementsList.add("motorVoltage");
//        measurementsList.add("motorCurrent");
//        measurementsList.add("remainingBattery");
//        measurementsList.add("maximumFeedbackPower");
//        measurementsList.add("maximumDischargePower");
//        measurementsList.add("selfCheckCounter");
//        measurementsList.add("totalBatteryCurrent");
//        measurementsList.add("totalBatteryVoltage");
//        measurementsList.add("singleBatteryMaxVoltage");
//        measurementsList.add("singleBatteryMinVoltage");
//        measurementsList.add("singleBatteryMaxTemperature");
//        measurementsList.add("singleBatteryMinTemperature");
//        measurementsList.add("availableBatteryCapacity");
//        measurementsList.add("vehicleStatus");
//        measurementsList.add("chargingStatus");
//        measurementsList.add("operatingStatus");
//        measurementsList.add("socStatus");
//        measurementsList.add("chargingEnergyStorageStatus");
//        measurementsList.add("driveMotorStatus");
//        measurementsList.add("positionStatus");
//        measurementsList.add("easStatus");
//        measurementsList.add("ptcStatus");
//        measurementsList.add("epsStatus");
//        measurementsList.add("absStatus");
//        measurementsList.add("mcuStatus");
//        measurementsList.add("heatingStatus");
//        measurementsList.add("batteryStatus");
//        measurementsList.add("batteryInsulationStatus");
//        measurementsList.add("dcdcStatus");
//        measurementsList.add("chgStatus");
        return measurementsList;
    }

    public List<String> getInsertValueList(JSONObject jsonObject,List<AnalyzeRule> analyzeRules) {
        List<String> valuesList = new ArrayList<>();

        for (AnalyzeRule analyzeRule : analyzeRules) {
            valuesList.add(jsonObject.getString(analyzeRule.getFieldsName()));
        }
//        valuesList.add(jsonObject.getString("startTime"));
//        valuesList.add(jsonObject.getString("longitude"));
//        valuesList.add(jsonObject.getString("latitude"));
//        valuesList.add(jsonObject.getString("speed"));
//        valuesList.add(jsonObject.getString("mileage"));
//        valuesList.add(jsonObject.getString("voltage"));
//        valuesList.add(jsonObject.getString("current"));
//        valuesList.add(jsonObject.getString("resistance"));
//        valuesList.add(jsonObject.getString("gear"));
//        valuesList.add(jsonObject.getString("accelerationPedal"));
//        valuesList.add(jsonObject.getString("brakePedal"));
//        valuesList.add(jsonObject.getString("fuelConsumptionRate"));
//        valuesList.add(jsonObject.getString("motorControllerTemperature"));
//        valuesList.add(jsonObject.getString("motorSpeed"));
//        valuesList.add(jsonObject.getString("motorTorque"));
//        valuesList.add(jsonObject.getString("motorTemperature"));
//        valuesList.add(jsonObject.getString("motorVoltage"));
//        valuesList.add(jsonObject.getString("motorCurrent"));
//        valuesList.add(jsonObject.getString("remainingBattery"));
//        valuesList.add(jsonObject.getString("maximumFeedbackPower"));
//        valuesList.add(jsonObject.getString("maximumDischargePower"));
//        valuesList.add(jsonObject.getString("selfCheckCounter"));
//        valuesList.add(jsonObject.getString("totalBatteryCurrent"));
//        valuesList.add(jsonObject.getString("totalBatteryVoltage"));
//        valuesList.add(jsonObject.getString("singleBatteryMaxVoltage"));
//        valuesList.add(jsonObject.getString("singleBatteryMinVoltage"));
//        valuesList.add(jsonObject.getString("singleBatteryMaxTemperature"));
//        valuesList.add(jsonObject.getString("singleBatteryMinTemperature"));
//        valuesList.add(jsonObject.getString("availableBatteryCapacity"));
//        valuesList.add(jsonObject.getString("vehicleStatus"));
//        valuesList.add(jsonObject.getString("chargingStatus"));
//        valuesList.add(jsonObject.getString("operatingStatus"));
//        valuesList.add(jsonObject.getString("socStatus"));
//        valuesList.add(jsonObject.getString("chargingEnergyStorageStatus"));
//        valuesList.add(jsonObject.getString("driveMotorStatus"));
//        valuesList.add(jsonObject.getString("positionStatus"));
//        valuesList.add(jsonObject.getString("easStatus"));
//        valuesList.add(jsonObject.getString("ptcStatus"));
//        valuesList.add(jsonObject.getString("epsStatus"));
//        valuesList.add(jsonObject.getString("absStatus"));
//        valuesList.add(jsonObject.getString("mcuStatus"));
//        valuesList.add(jsonObject.getString("heatingStatus"));
//        valuesList.add(jsonObject.getString("batteryStatus"));
//        valuesList.add(jsonObject.getString("batteryInsulationStatus"));
//        valuesList.add(jsonObject.getString("dcdcStatus"));
//        valuesList.add(jsonObject.getString("chgStatus"));
        return valuesList;
    }

}
