package com.hyc.kafka.demo.listener;

import com.alibaba.fastjson.JSONObject;
import com.hyc.config.RabbitmqConfig;
import com.hyc.domain.FaultMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.listener.KeyExpirationEventMessageListener;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * key过期监听器
 *
 * @author YouChe·He
 * @ClassName: KeyExpiredListener
 * @Description: key过期监听器
 * @CreateTime: 2024/6/20 21:59
 */
@Slf4j
public class KeyExpiredListener extends KeyExpirationEventMessageListener {

    private RabbitTemplate rabbitTemplate;

    public KeyExpiredListener(RedisMessageListenerContainer listenerContainer,RabbitTemplate rabbitTemplate) {
        super(listenerContainer);
        this.rabbitTemplate = rabbitTemplate;
    }

    @Override
    public void onMessage(Message message, byte[] pattern) {
        String expireKey = message.toString();
        log.info("过期的key：" + expireKey);
        //在这里做过期key的处理
        if (expireKey.startsWith("car_")){
            String[] split = expireKey.split("_");
            FaultMessage faultMessage = new FaultMessage(split[1], split[2], new Date(), "update");
            rabbitTemplate.convertAndSend(RabbitmqConfig.FAULT_EXCHANGE,RabbitmqConfig.FAULT_MESSAGE_ROUTINGKEY, JSONObject.toJSONString(faultMessage));
        }
    }
}
