package com.hyc.kafka.demo.strategy;

import com.alibaba.fastjson.JSONObject;
import com.hyc.config.RabbitmqConfig;
import com.hyc.domain.VehicleData;
import com.hyc.domain.cache.CacheIndexWarn;
import com.hyc.domain.cache.IndexWarnToRedis;
import com.hyc.domain.constant.EventConstant;
import com.hyc.domain.resp.IndexWarnEventResp;
import com.hyc.iotdbdemo.model.param.IotDbParam;
import com.hyc.iotdbdemo.server.IotDbServer;
import com.hyc.util.EventHandleUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 事件策略枚举类
 *
 * @author YouChe·He
 * @ClassName: StrategyA
 * @Description: 策略枚举A
 * @CreateTime: 2024/6/4 14:13
 */
@Slf4j
@Component
public enum EventStrategy {


    /**
     * 实时数据
     */
    REAL_TIME_DATA {
        @Override
        public void exe(CacheManager cacheManager, RedisTemplate<String, String> redisTemplate, JSONObject jsonObject, RabbitTemplate rabbitTemplate, ApplicationEventPublisher applicationEventPublisher, ScheduledExecutorService executorService, IotDbServer iotDbServer) {
            log.info("执行实时数据逻辑");

            if (redisTemplate.hasKey(jsonObject.getString("vin") + EventConstant.REAL_TIME_DATA)) {
                //有VIN_REAL_TIME_DATA这个key,则代表用户有查询对应车辆VIN数据的请求,将数据存储到redis中
                String key = jsonObject.getString("vin") + EventConstant.REAL_TIME_DATA;
                // 获取当前的过期时间
                Long currentTTL = redisTemplate.getExpire(key, TimeUnit.SECONDS);
                // 更新键的值
                redisTemplate.opsForList().rightPush(key, jsonObject.toJSONString());
                // 重新设置过期时间
                if (currentTTL != null && currentTTL > 0) {
                    redisTemplate.expire(key, currentTTL, TimeUnit.SECONDS);
                }

            }
        }
    },
    /**
     * 故障报警
     */
    FAULT_ALARM {
        @Override
        public void exe(CacheManager cacheManager, RedisTemplate<String, String> redisTemplate, JSONObject jsonObject, RabbitTemplate rabbitTemplate, ApplicationEventPublisher applicationEventPublisher, ScheduledExecutorService executorService, IotDbServer iotDbServer) {
            System.out.println("执行故障报警逻辑");


            if (jsonObject.getInteger("vehicleStatus") != null && jsonObject.getInteger("vehicleStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ001", rabbitTemplate);
            }

            if (jsonObject.getInteger("chargingStatus") != null && jsonObject.getInteger("chargingStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ002", rabbitTemplate);
            }

            if (jsonObject.getInteger("operatingStatus") != null && jsonObject.getInteger("operatingStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ003", rabbitTemplate);
            }

            if (jsonObject.getInteger("socStatus") != null && jsonObject.getInteger("socStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ004", rabbitTemplate);
            }

            if (jsonObject.getInteger("chargingEnergyStorageStatus") != null && jsonObject.getInteger("chargingEnergyStorageStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ005", rabbitTemplate);
            }

            if (jsonObject.getInteger("driveMotorStatus") != null && jsonObject.getInteger("driveMotorStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ006", rabbitTemplate);
            }

            if (jsonObject.getInteger("positionStatus") != null && jsonObject.getInteger("positionStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ007", rabbitTemplate);
            }

            if (jsonObject.getInteger("easStatus") != null && jsonObject.getInteger("easStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ008", rabbitTemplate);
            }

            if (jsonObject.getInteger("ptcStatus") != null && jsonObject.getInteger("ptcStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ009", rabbitTemplate);
            }

            if (jsonObject.getInteger("epsStatus") != null && jsonObject.getInteger("epsStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ010", rabbitTemplate);
            }

            if (jsonObject.getInteger("absStatus") != null && jsonObject.getInteger("absStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ011", rabbitTemplate);
            }

            if (jsonObject.getInteger("mcuStatus") != null && jsonObject.getInteger("mcuStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ012", rabbitTemplate);
            }

            if (jsonObject.getInteger("heatingStatus") != null && jsonObject.getInteger("heatingStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ013", rabbitTemplate);
            }

            if (jsonObject.getInteger("batteryStatus") != null && jsonObject.getInteger("batteryStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ014", rabbitTemplate);
            }

            if (jsonObject.getInteger("batteryInsulationStatus") != null && jsonObject.getInteger("batteryInsulationStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ015", rabbitTemplate);
            }

            if (jsonObject.getInteger("dcdcStatus") != null && jsonObject.getInteger("dcdcStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ016", rabbitTemplate);
            }

            if (jsonObject.getInteger("chgStatus") != null && jsonObject.getInteger("chgStatus") == 0) {
                EventHandleUtil.insertToCache(cacheManager, redisTemplate, jsonObject, "GZ017", rabbitTemplate);
            }

        }
    },
    /**
     * 电子围栏
     */

    ELECTRONIC_FENCE {
        @Override
        public void exe(CacheManager cacheManager, RedisTemplate<String, String> redisTemplate, JSONObject jsonObject, RabbitTemplate rabbitTemplate, ApplicationEventPublisher applicationEventPublisher, ScheduledExecutorService executorService, IotDbServer iotDbServer) {

            log.info("执行电子围栏逻辑");
        }
    },
    /**
     * 指标预警
     */
    INDEX_WARNING {
        @Override
        public void exe(CacheManager cacheManager, RedisTemplate<String, String> redisTemplate, JSONObject jsonObject, RabbitTemplate rabbitTemplate, ApplicationEventPublisher applicationEventPublisher, ScheduledExecutorService executorService, IotDbServer iotDbServer) {
            String vin = jsonObject.getString("vin");

            Cache indexWarn = cacheManager.getCache("indexWarn");
            Cache.ValueWrapper valueWrapper = indexWarn.get(vin + "_Index_Warn");
            if (valueWrapper == null){
                String indexWarnJsonString = redisTemplate.opsForValue().get(vin + "_Index_Warn");

                indexWarn.put(vin+"_Index_Warn", indexWarnJsonString);
                Cache indexWarnTwo = cacheManager.getCache("indexWarn");
                Cache.ValueWrapper valueWrapperTwo = indexWarn.get(vin + "_Index_Warn");
                String cacheIndexWarnJsonString = valueWrapperTwo.get().toString();
                IndexWarnToRedis cacheIndexWarn = JSONObject.parseObject(cacheIndexWarnJsonString, IndexWarnToRedis.class);
                log.info("指标预警Boot事件首次发布!");
                applicationEventPublisher.publishEvent(new CacheIndexWarn(this, cacheIndexWarn.getVin(), cacheIndexWarn.getIndexNames(), cacheIndexWarn.getTotalLength(), cacheIndexWarn.getSlideLength()));
                log.warn("进来了吗？");

            }else {
                log.info("车辆:{}对应指标预警事件正在执行!",vin);
            }


        }
    };


    public abstract void exe(CacheManager cacheManager, RedisTemplate<String, String> redisTemplate, JSONObject jsonObject, RabbitTemplate rabbitTemplate, ApplicationEventPublisher applicationEventPublisher, ScheduledExecutorService executorService, IotDbServer iotDbServer);

}
