package com.hyc.localevent.listener;

import com.alibaba.fastjson.JSONObject;
import com.hyc.config.RabbitmqConfig;
import com.hyc.domain.cache.CacheIndexWarn;
import com.hyc.domain.cache.IndexWarnToRedis;
import com.hyc.domain.resp.IndexWarnEventResp;
import com.hyc.iotdbdemo.model.param.IotDbParam;
import com.hyc.iotdbdemo.server.IotDbServer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 指标预警监听者
 *
 * @author YouChe·He
 * @ClassName: IndexWarnListener
 * @Description: 指标预警监听者
 * @CreateTime: 2024/6/28 16:06
 */
@Slf4j
@Component
public class IndexWarnListener {

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    @Autowired
    private CacheManager cacheManager;

    @Autowired
    private RabbitTemplate rabbitTemplate;
    @Autowired
    private ScheduledExecutorService executorService;
    @Autowired
    private IotDbServer iotDbServer;


    @EventListener
    public void handleIndexWarnEvent(CacheIndexWarn cacheIndexWarn){

        executorService.schedule(() -> {
            long time = new Date().getTime();

            try {
                HashMap<String, Double> stringDoubleHashMap = new HashMap<>();
                //根据VIN从iotDb获取数据
                List<String> carDataMapList = (List<String>) iotDbServer.queryDataFromIotDb(new IotDbParam(cacheIndexWarn.getVin(), String.valueOf(time - cacheIndexWarn.getSlideLength() * 1000), String.valueOf(time)));


                //获取该VIN对应的指标预警配置
                String[] split = cacheIndexWarn.getIndexNames().split(",");
                for (String s : carDataMapList) {
                    for (String s1 : split) {
                        log.warn("s:{},s1:{}",s,s1);
                        String string = JSONObject.parseObject(s).getString(s1);
                        stringDoubleHashMap.put(s1,stringDoubleHashMap.get(s1)==null?Double.valueOf(string):Double.valueOf(string)+stringDoubleHashMap.get(s1));
                    }
                }
                for (String s : split) {
                    stringDoubleHashMap.put(s,stringDoubleHashMap.get(s) / split.length);
                }

                ArrayList<IndexWarnEventResp> indexWarnEventRespList = new ArrayList<>();
                for (String s : stringDoubleHashMap.keySet()) {
                    String qs = "";
                    if (stringDoubleHashMap.get(s)<600){
                        qs = "下降";
                    } else if (stringDoubleHashMap.get(s) > 8000) {
                        qs = "上升";
                    }else {
                        qs = "波动";
                    }
                    indexWarnEventRespList.add(new IndexWarnEventResp(cacheIndexWarn.getVin(),s,stringDoubleHashMap.get(s),qs));
                }


                rabbitTemplate.convertAndSend(RabbitmqConfig.INDEX_WARN_QUEUE, JSONObject.toJSONString(indexWarnEventRespList));
                log.error("计算出来的值是:{}",stringDoubleHashMap);

                Cache indexWarn = cacheManager.getCache("indexWarn");
                Cache.ValueWrapper valueWrapper = indexWarn.get(cacheIndexWarn.getVin() + "_Index_Warn");
                if (valueWrapper != null){
                    String cacheIndexWarnJsonStringTwo = valueWrapper.get().toString();
                    IndexWarnToRedis cacheIndexWarnTwo = JSONObject.parseObject(cacheIndexWarnJsonStringTwo, IndexWarnToRedis.class);

                    applicationEventPublisher.publishEvent(new CacheIndexWarn(this, cacheIndexWarnTwo.getVin(), cacheIndexWarnTwo.getIndexNames(), cacheIndexWarnTwo.getTotalLength(), cacheIndexWarnTwo.getSlideLength()));

                }else {
                    log.info("车辆:{}已下线,指标预警停止",cacheIndexWarn.getVin());
                }

            } catch (Exception e) {
                throw new RuntimeException(e);
            }

        }, cacheIndexWarn.getSlideLength(), TimeUnit.SECONDS);

        log.error("几个意思,得到的值是啥:{}",cacheIndexWarn);
    }






}
