package com.hyc.util;

import com.alibaba.fastjson.JSONObject;
import com.hyc.config.RabbitmqConfig;
import com.hyc.domain.FaultMessage;
import com.hyc.domain.VehicleData;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 事件处理工具类
 *
 * @author YouChe·He
 * @ClassName: EventHandleUtil
 * @Description: 事件处理工具类
 * @CreateTime: 2024/6/20 15:55
 */
@Slf4j
public class EventHandleUtil {
    public static void insertToCache(CacheManager cacheManager, RedisTemplate<String,String> redisTemplate, JSONObject jsonObject, String keySuffix, RabbitTemplate rabbitTemplate){
        Cache faultMessageCache = cacheManager.getCache("faultMessage");
        Cache.ValueWrapper valueWrapper = faultMessageCache.get("car_"+jsonObject.getString("vin") + "_"+keySuffix);
        if (valueWrapper == null){
            FaultMessage faultMessage = new FaultMessage(jsonObject.getString("vin"), keySuffix, new Date(), "insert");
            //发送故障开始消息
            log.error("发送故障消息!!!");
            rabbitTemplate.convertAndSend(RabbitmqConfig.FAULT_EXCHANGE,RabbitmqConfig.FAULT_MESSAGE_ROUTINGKEY,JSONObject.toJSONString(faultMessage));
            faultMessageCache.put("car_"+jsonObject.getString("vin") + "_" + keySuffix, jsonObject.toJSONString());
            redisTemplate.opsForValue().set("car_"+jsonObject.getString("vin")+"_"+keySuffix,jsonObject.toJSONString(),10, TimeUnit.SECONDS);
        }else {
            log.warn("已经存在{}故障",keySuffix);
            //已经存在此故障,继续添加 再次添加应该会更新缓存时间
            faultMessageCache.put("car_"+jsonObject.getString("vin") + "_" + keySuffix, jsonObject.toJSONString());
            //同时添加到redis,设置过期时间为10S
            redisTemplate.opsForValue().set("car_"+jsonObject.getString("vin")+"_"+keySuffix,jsonObject.toJSONString(),10, TimeUnit.SECONDS);
        }
    }
}
